#!/bin/bash
################################################################################
#                                                                              #
# Copyright (c) 2019 Oracle and/or its affiliates. All rights reserved.        #
#                                                                              #
################################################################################
#
# Single entrypoint to setup a bastion host for OCCNE on Bare Metal
#
# Define (some variables are derived from the hosts.ini file):
#   OCCNE_CLUSTER      - OCCNE cluster name                 - Derived
#   OCCNE_HOSTS_INI    - Path to hosts.ini                  - default /var/occne/cluster/${OCCNE_CLUSTER}/hosts.ini
#   OCCNE_PREFIX       - Possible prefix for images         - Default not set
#   OCCNE_VERSION      - Image version                      - Default 1.3.1
#   OCCNE_USER         - OCCNE node access user             - Default admusr
#   OCCNE_SKIP_BASTION - Skip Bastion Host setup            - Default 0 (no)
#   CENTRAL_REPO       - hostname of central repo server    - Default winterfell
#   CENTRAL_REPO_IP    - IP address of central repo server  - Default 10.75.216.10
#   CENTRAL_REPO_DOCKER_PORT - Central repo port number     - Default 5000
#   OCCNE_BASTION      - Bastion Host name                  - Default bastion-2

# Enable all executed commands are output to the terminal.
set -x

# Path to the hosts.ini file. Defaults to /var/occne/cluster/${OCCNE_CLUSTER}/hosts.ini.
#
echo OCCNE_HOSTS_INI=${OCCNE_HOSTS_INI:=/var/occne/cluster/${OCCNE_CLUSTER}/hosts.ini}
[ "${OCCNE_HOSTS_INI}" ] || exit 1

# Get the cluster name from the hosts.ini file. Assumes hosts.ini file exists.
#
echo OCCNE_CLUSTER=${OCCNE_CLUSTER:=$(`cat "${OCCNE_HOSTS_INI}" | grep occne_cluster_name | cut -d"=" -f2 | tr -d '"'`)}
[ "${OCCNE_CLUSTER}" ] || exit 1

# User supplied bastion node. User only supplies this if the
# Bastion Host targeted is not bastion-2
#
echo OCCNE_BASTION=${OCCNE_BASTION:=bastion-2}
[ "${OCCNE_BASTION}" ] || exit 1

# Path to configBastionHost.yaml playbook. Defaults to
# /var/occne/configBastionHost.yaml.
#
echo OCCNE_CONFIG_FILE=${OCCNE_CONFIG_FILE:=/var/occne/configBastionHost.yaml}
[ "${OCCNE_CONFIG_FILE}" ] || exit 1

# Possible prefix for images (usually undefined)
#
echo OCCNE_PREFIX=${OCCNE_PREFIX}

# OCCNE Images version tag
#
echo OCCNE_VERSION=${OCCNE_VERSION:=1.3.1}
[ "${OCCNE_VERSION}" ] || exit 1

# Customer supplied repo name. Defaults to winterfell.
#
echo CENTRAL_REPO=${CENTRAL_REPO:=winterfell}
[ "${CENTRAL_REPO}" ] || exit 1

# Customer supplied repo IPv4 address. Default to winterfell
# IPv4.
#
echo CENTRAL_REPO_IP=${CENTRAL_REPO_IP:=10.75.216.10}
[ "${CENTRAL_REPO_IP}" ] || exit 1

# Customer supplied repo port number. Defaults to
# port number 5000.
#
echo CENTRAL_REPO_DOCKER_PORT=${CENTRAL_REPO_DOCKER_PORT:=5000}
[ "${CENTRAL_REPO_DOCKER_PORT}" ] || exit 1

# Current occne user. Defaults to admusr.
#
echo OCCNE_USER=${OCCNE_USER:=admusr}
[ "${OCCNE_USER}" ] || exit 1

# This is mainly for test purposes should a developer
# need to set the pipeline args themself.
#
echo OCCNE_PIPELINE_ARGS=${OCCNE_PIPELINE_ARGS}


# Install ansible. This requires that this server has access to the
# necessary yum mirrors as defined by the customer. If we cannot
# install ansible then there is no need to proceed.
#
sudo yum install -y ansible
if [ $? -eq 0 ]; then
    # Since the below commands pipe out to tee, we really want to find
    # failures on ANY command of the pipe, not just the 'tee'.
    #
    set -o pipefail

    # Configure the bastion host. If skipping bastion-host re-deployment,
    # assume it is already configured and just update it with potentially
    # new cluster inventory.
    #
    if [ "${OCCNE_SKIP_BASTION}" ]; then
        CONFIG_LIMIT='--tags=inventory_update'
    fi

    # Execute the configBastionHost.yaml ansible playbook to pre-configure
    # the Bastion Host.
    #
    ansible-playbook --become -i ${OCCNE_HOSTS_INI} --extra-vars="ansible_ssh_private_key_file=/var/occne/cluster/${OCCNE_CLUSTER}/.ssh/occne_id_rsa ansible_user=${OCCNE_USER} central_repo_hostname=${CENTRAL_REPO} central_repo_ip=${CENTRAL_REPO_IP} central_repo_docker_port=${CENTRAL_REPO_DOCKER_PORT} image_version=${OCCNE_VERSION} ${OCCNE_PREFIX:+image_prefix=${OCCNE_PREFIX}/} occne_cluster_name=${OCCNE_CLUSTER} occne_bastion_host_name=${OCCNE_BASTION}" ${CONFIG_LIMIT} ${OCCNE_CONFIG_FILE} | tee bastion$(date -u +"%F_%H%MZ").log

    if [ $? -eq 0 ]; then
        # On success of bastion bootstrap, fire off the pipeline
        # (first will configure bastion, probably rebooting it).
        #
        PIPELINE_ARGS="${OCCNE_PIPELINE_ARGS} OCCNE_VERSION=${OCCNE_VERSION} ${OCCNE_PREFIX:+OCCNE_PREFIX=${OCCNE_PREFIX}}"

        [ "${OCCNE_SKIP_BASTION}" ] || ssh -t -t -i /var/occne/cluster/${OCCNE_CLUSTER}/.ssh/occne_id_rsa ${OCCNE_USER}@${OCCNE_BASTION} "SETUP_BASTION=ONLY ${PIPELINE_ARGS} /var/occne/cluster/${OCCNE_CLUSTER}/artifacts/pipeline.sh" | tee pipeline$(date +"%F_%H%M%S").log
        if [ $? -eq 255 ]; then
           echo "Bastion host had to reboot due to updates. Please wait until it completes the reboot before logging into the Bastion Host."
        fi
    else
        echo "ERROR $?: Bastion setup issue - pipeline script failure"
    fi

    # Restore pipefail to expected value.
    #
    set +o pipefail
else
    echo "ERROR $?: Ansible installation failure"
    exit 1
fi
